#!/usr/bin/python

# NetworkDesign version number
__version__ = "0.1"

import sys
import getopt
from mod_solver import ModSolver
from mod_network import ModNetwork
from mod_models import ModSecDesignModel
from mod_solution import ModSolution


def usage():

    print "ERROR: Not enough or invalid parameters"
    print "Usage: ./NetworkDesign.py [-i(--input=) nsfile.ns] [-o(--output=) solution.graphml] -s(--scip=) path-to-scip-solver -h(--help)"
    print "\t -i(--input=): the ns-2 file that includes the complete description of the network and of the optimization problem's parameters (optional)"
    print "\t -o(--output=): the name of the output GraphML file (optional)"
    print "\t -s(--scip=): the path to the SCIP solver (example: /home/user/scipoptsuite-3.2.0/scip-3.2.0/bin/scip)"
    print "\t -h(--help): prints this help message"
    sys.exit(0)


if __name__ == '__main__':

    # Process arguments
    try:
        opts, args = getopt.getopt(sys.argv[1:], "h:v,i:v,o:v,s:v", ["help", "input=", "output=", "scip="])
    except getopt.GetoptError as err:
        usage()
        sys.exit(0)

    inputFile = 'network.ns'
    outputFile = 'solution.graphml'
    scipPath = None
    for o, a in opts:
        if o in ("-h", "--help"):
            usage()
            sys.exit()
        elif o in ("-i", "--input"):
            print "Found input:", a
            inputFile = a
        elif o in ("-o", "--output"):
            print "Found output:", a
            outputFile = a
        elif o in ("-s", "--scip"):
            print "Found scip:", a
            scipPath = a
        else:
            assert False, "unhandled option"

    if scipPath == None:
        usage()
        sys.exit(0)

    print "NetworkDesign version: ", __version__
    print "\t Input file: ", inputFile
    print "\t Output file: ", outputFile
    print "\t Path to SCIP: ", scipPath

    netmod = ModSecDesignModel(inputFile)
    solver = ModSolver(scipPath)

    # Parse the input NS file and generate the LP file
    netmod.parseNSFile()
    netmod.generateLPFile()
   
    # Solve the optimization problem
    sol = solver.solve()
    print "*** The solution of the optimization is: ", sol

    # Get the solution mapping
    solmap = netmod.getSolutionMapping(sol)
    print "*** The mapping of the solution: ", solmap

    # Generate the output file
    solwr = ModSolution(outputFile)
    solwr.writeSolution(solmap)

    print "*** The solution was written to: ", outputFile
